local Skada = Skada

Skada:AddLoadableModule("Multistrike", function(Skada, L)
	if Skada.db.profile.modulesBlocked.Multistrike then return end
	
	local mod = Skada:NewModule("Multistrike damage")
	local playerMulti = Skada:NewModule("Multistrike spell list")
	local spellsMulti = Skada:NewModule("Multistrike spell details")
	local multiTarget = Skada:NewModule("Multistrike hit mobs")

	local sourceDmg
	local function getDPS(set, player, sourceDmg)
		local totaltime = Skada:PlayerActiveTime(set, player)

		return sourceDmg / math.max(1, totaltime)
	end

	local function getRaidMultiDPS(set)
		if set.time > 0 then
			return set.multiDamage / math.max(1, set.time)
		else
			local endtime = set.endtime
			if not endtime then
				endtime = time()
			end
			return set.multiDamage / math.max(1, endtime - set.starttime)
		end
	end

	local function log_damage(set, multi)
		-- Get the player.
		local player = Skada:get_player(set, multi.playerid, multi.playername)
		if player then

			-- Subtract overkill
	--		local amount = math.max(0,multi.amount - multi.overkill)
			-- Or don't. Seems to be the way other meters do it.
			local amount = multi.amount
	--		self:Print(player.name..": "..multi.spellname.." for "..tostring(amount))

			-- Also add to set total and total non multistrike damage. NOT USED
			-- set.nonMultiDamage = set.nonMultiDamage + amount		

			-- Add spell to player if it does not exist.
			-- Add total multistrikes if not exist
			if not player.multiDamageSpells[multi.spellname] then
				player.multiDamageSpells[multi.spellname] = {id = multi.spellid, totalHits = 0, damage = 0, totalMultis = 0, multiDamage = 0}
			end
			
			-- Get the spell from player.
			local spell = player.multiDamageSpells[multi.spellname]
			
			-- Add to player total damage.
			player.nonMultiDamage = player.nonMultiDamage + amount
			spell.damage = spell.damage + amount
			
			player.totalHits = player.totalHits + 1
			spell.totalHits = spell.totalHits + 1
			
			if multi.multistrike then
				spell.totalMultis = spell.totalMultis + 1
				spell.multiDamage = spell.multiDamage + amount
				
				player.totalMultis = player.totalMultis + 1
				player.multiDamage = player.multiDamage + amount
				
				set.multiDamage = set.multiDamage + amount
				
				if spell.max == nil or amount > spell.max then
					spell.max = amount
				end
				
				if (spell.min == nil or amount < spell.min) and not multi.missed then
					spell.min = amount
				end
			end
			
			-- For now, only save multiDamage info to current set.
			-- Saving this to Total may become a memory hog deluxe, and besides, it does not make much sense
			-- to see in Total. Why care which particular mob you multiDamage the most in a whole raid, for example?
			if set == Skada.current and multi.dstname and amount > 0 and multi.multistrike then
				-- Make sure destination exists in player.
				if not player.multiDamaged[multi.dstname] then
					player.multiDamaged[multi.dstname] = 0
				end

				-- Add to destination.
				player.multiDamaged[multi.dstname] = player.multiDamaged[multi.dstname] + amount
			end

		end
	end
		
	local multi = {}

	local function SwingDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, smultistrike)
		-- White melee.
		if srcGUID ~= dstGUID then
			multi.playerid = srcGUID
			multi.playername = srcName
			multi.playerflags = srcFlags
			multi.dstname = dstName
			multi.spellid = 6603
			multi.spellname = "Attack"
			multi.amount = samount
			multi.multistrike = smultistrike
			multi.missed = nil
			
			Skada:FixPets(multi)
			log_damage(Skada.current, multi)
			log_damage(Skada.total, multi)
		end
	end
		
	local function SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, smultistrike)
	-- Spell damage.
		if srcGUID ~= dstGUID then
			-- XXX WoD quick fix for Mage's Prismatic Crystal talent
			-- All damage done to the crystal is transferred, so ignore it
			if dstGUID:match("^Creature%-0%-%d+%-%d+%-%d+%-76933%-%w+$") then
				return
			end
			
			-- XXX Filter spells with 100% multi chance
			-- Not worth while to include, skews everything
			-- if spellId ~= 5176 then
				-- return
			-- end
			
			multi.playerid = srcGUID
			multi.playerflags = srcFlags
			multi.dstname = dstName
			multi.playername = srcName
			multi.spellid = spellId
			multi.spellname = spellName
			multi.amount = samount
			multi.multistrike = smultistrike
			multi.missed = nil
			
			Skada:FixPets(multi)
			log_damage(Skada.current, multi)
			log_damage(Skada.total, multi)
		end
	end

	function mod:Update(win, set)
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.nonMultiDamage > 0 then
				local multiDps = getDPS(set, player, player.multiDamage)

				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name

				d.value = player.multiDamage
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				d.valuetext = Skada:FormatValueText(
					Skada:FormatNumber(player.multiDamage), self.metadata.columns.Damage,
					Skada:FormatNumber(multiDps), self.metadata.columns.DPS,
					string.format("%02.1f%%", (player.multiDamage / set.multiDamage) * 100), self.metadata.columns.Percent
				)
				if player.multiDamage > max then
					max = player.multiDamage
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function player_tooltip(win, id, label, tooltip)
		local player = Skada:find_player(win:get_selected_set(), playerMulti.playerid)
		if player then
			local spell = player.multiDamageSpells[label]
			if spell then
				tooltip:AddLine(player.name.." - "..label)
				if spell.max and spell.min then
					tooltip:AddDoubleLine(L["Minimum hit:"], Skada:FormatNumber(spell.min), 255,255,255,255,255,255)
					tooltip:AddDoubleLine(L["Maximum hit:"], Skada:FormatNumber(spell.max), 255,255,255,255,255,255)
				end
				tooltip:AddDoubleLine(L["Average hit:"], Skada:FormatNumber(spell.multiDamage / spell.totalMultis), 255,255,255,255,255,255)
			end
		end
	end
	
	function playerMulti:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			playerMulti.playerid = id
			playerMulti.title = player.name.."'s Multistrike Damage"
		end
	end

	function playerMulti:Update(win,set)
		-- View spells for this player.
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for spellname, spell in pairs(player.multiDamageSpells) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = spellname
					d.id = spellname
					local _, _, icon = GetSpellInfo(spell.id)
					d.icon = icon
					d.spellid = spell.id
					d.value = spell.multiDamage
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(spell.multiDamage), self.metadata.columns.Damage,
						string.format("%02.1f%%", (spell.multiDamage / player.multiDamage) * 100), self.metadata.columns.Percent
					)
					if spell.multiDamage > max then
						max = spell.multiDamage
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value)
		local nr = spellsMulti.nr + 1
		spellsMulti.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		d.valuetext = Skada:FormatValueText(
			value, mod.metadata.columns.Damage,
			string.format("%02.1f%%", value / spellsMulti.totalHits * 100), mod.metadata.columns.Percent
		)

		win.metadata.maxvalue = math.max(win.metadata.maxvalue, value)
	end
	
	function spellsMulti:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playerMulti.playerid)
		spellsMulti.spellname = label
		spellsMulti.title = player.name.."'s "..label
	end
	--
	function spellsMulti:Update(win, set)
		local player = Skada:find_player(set, playerMulti.playerid)

		if player then
		local spell = player.multiDamageSpells[self.spellname]

			if spell then
				spellsMulti.totalHits = spell.totalHits
				spellsMulti.nr = 0
				win.metadata.maxvalue = 0
				
				if spell.totalHits and spell.totalHits > 0 then
					add_detail_bar(win, "Total", spell.totalHits)
				end
				if spell.totalMultis and spell.totalMultis > 0 then
					add_detail_bar(win, "Multistrike", spell.totalMultis)
				end
			end
		end
	end

	function multiTarget:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		multiTarget.playerid = id
		multiTarget.title = (player and player.name or "Unknown").."'s Multistrike hit mobs"

	end
--
	function multiTarget:Update(win, set)
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for mob, amount in pairs(player.multiDamaged) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = mob
					d.id = mob
					d.value = amount
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(amount), self.metadata.columns.Damage,
						string.format("%02.1f%%", (amount / player.multiDamage) * 100), self.metadata.columns.Percent
					)
					if amount > max then
						max = amount
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = playerMulti, click2 = multiTarget, columns = {Damage = true, DPS = true, Percent = true}}
		playerMulti.metadata	= {tooltip = player_tooltip, click1 = spellsMulti, columns = {Damage = true, Percent = true}}
		spellsMulti.metadata = {columns = {Damage = true, Percent = true}}
		multiTarget.metadata = {showspots = true, columns = {Damage = true, Percent = true}}
			

		Skada:RegisterForCL(SwingDamage, 'SWING_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		
		Skada:RegisterForCL(SpellDamage, 'DAMAGE_SHIELD', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_PERIODIC_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_BUILDING_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'RANGE_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.multiDamageSpells or not player.multiDamage or not player.totalMultis or not player.totalHits or not player.nonMultiDamage then
			player.multiDamageSpells = player.multiDamageSpells or {}		-- Stored spell multistrike info
			player.multiDamage = player.multiDamage or 0					-- Total multistrike damage
			player.totalMultis = player.totalMultis or 0
			player.totalHits = player.totalHits or 0
			player.nonMultiDamage = player.nonMultiDamage or 0
		end
		
		if not player.multiDamaged then
			player.multiDamaged = {}
		end
	end

	function mod:AddToTooltip(set, tooltip)
		GameTooltip:AddDoubleLine("Multistrike DPS", Skada:FormatNumber(getRaidMultiDPS(set)), 1,1,1)
	end

	function mod:AddSetAttributes(set)
		if not set.nonMultiDamage then
			set.nonMultiDamage = 0
		end
		if not set.multiDamage then
			set.multiDamage = 0
		end
	end

	function mod:GetSetSummary(set)
		return Skada:FormatValueText(
			Skada:FormatNumber(set.multiDamage), self.metadata.columns.Damage,
			Skada:FormatNumber(getRaidMultiDPS(set)), self.metadata.columns.DPS
		)
	end
end)